/*
 * boom.c - BOOM, main function
 *
 * Copyright 2012 by Werner Almesberger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "util.h"
#include "lang.h"
#include "param.h"
#include "subex.h"
#include "db.h"


static struct param *vars = NULL, **last_var = &vars;
static int select_parts = 0;


static void add_var(const char *arg)
{
	char *tmp = stralloc(arg);
	char *eq;

	eq = strchr(tmp, '=');
	if (!eq) {
		fprintf(stderr, "no = in variable setting\n");
		exit(1);
	}
	*eq = 0;
	*last_var = make_var(tmp, rel_eq, eq+1);
	last_var = &(*last_var)->next;
	free(tmp);
}


static void do_substitutions(void)
{
	struct param *out;
	const struct param *var;
	const struct part **parts, **p;

	if (!substitute(substitutions, vars, &out)) {
		fprintf(stderr, "ignore\n");
		return;
	}
	if (select_parts) {
		parts = select_parametric(out, &hierarchy);
		if (!parts) {
			fprintf(stderr, "no matches\n");
		} else {
			for (p = parts; *p; p++)
				printf("%s %s\n", (*p)->domain, (*p)->name);
			free(parts);
		}
	} else {
		for (var = out; var; var = var->next) {
			printf("%s", var->u.name);
			dump_relop(stdout, var->op);
			printf("%s\n", var->value.u.s);
		}
		free_vars(out);
	}
}


static void usage(const char *name)
{
	fprintf(stderr,
"usage: %s file [[-type] file ...] [-q var=value ...] ...\n\n"
"  file types:\n"
"  -c  characteristics\n"
"  -i  inventory\n"
"  -x  currency exchange\n"
"  -p  providers\n"
"  -s  substitutions\n"
"  -q var=value ...\n"
"  -Q var=value ...\n"
    , name);
	exit(1);
}


int main(int argc, char **argv)
{
	void (*process)(const char *name) = parse_hierarchy;
	int i;

	dollar = unique("$");
	for (i = 1; i != argc; i++) {
		if (*argv[i] == '-') {
			if (!strcmp(argv[i], "-c"))
				process = parse_characteristics;
			else if (!strcmp(argv[i], "-i"))
				process = parse_inventory;
			else if (!strcmp(argv[i], "-x"))
				process = parse_currencies;
			else if (!strcmp(argv[i], "-p"))
				process = parse_providers;
			else if (!strcmp(argv[i], "-s"))
				process = parse_substitutions;
			else if (!strcmp(argv[i], "-q"))
				process = add_var;
			else if (!strcmp(argv[i], "-Q")) {
				process = add_var;
				select_parts = 1;
			} else
				usage(*argv);
		} else {
			process(argv[i]);
		}
	}
	if (vars)
		do_substitutions();
	return 0;
}
